classdef GaussianBayesianNetwork < org.mensxmachina.pgm.BayesianNetwork
% Gaussian Bayesian Network

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

methods

function bnet = GaussianBayesianNetwork(G, Param, varargin)
%GAUSSIANBAYESIANNETWORK Create a Categorical Bayesian Network
%   A Gaussian Bayesian Network is a Bayesian Network whose all nodes
%   are linear Gaussian.
%
%   BNET = GAUSSIANBAYESIANNETWORK(G, NUMLEVELS, PARAM)
%   creates a Gaussian Bayesian Network with DAG G, node number of
%   levels NUMLEVELS and node parameters PARAM. G is a NxN sparse
%   connectivity matrix, where M is the number of nodes of the network.
%   PARAM is a cell array of length equal to the number of nodes. Each cell
%   contains a struct with fields ''mu'', ''b'' and ''sigma'' specifying
%   the mean, the regression coefficients and the standard deviation of the
%   corresponding node. MU and SIGMA must be real scalars. B must be a real
%   vector of length equal to the number of parents of the node.
%
%   BNET = GAUSSIANBAYESIANNETWORK(G, NUMLEVELS, PARAM, ...
%       'Param1', VAL1,'Param2', VAL2,...) specifies additional parameter 
%   name/value pairs of the BAYESIANNETWORK class.
%
%   See also ORG.MENSXMACHINA.PGM.BAYESIANNETWORK

% set all number of levels to missing
numLevels = repmat(NaN, 1, size(G, 1));

% set all conditional distribution function parameter - returning functions
% to GAUSSIANCPDFPARAM
CpdfParamFun = ...
    repmat({@gaussiancpdfparam}, 1, size(G, 1));

% set all node conditional distribution random number generator functions
% to 'norm'
CpdfRndFun = ...
    repmat({'norm'}, 1, size(G, 1));

% call superclass constructor
bnet = bnet@org.mensxmachina.pgm.BayesianNetwork( ...
    G, numLevels, Param, CpdfParamFun, CpdfRndFun, varargin{:});

% deep check PARAM
goodParam = ...
    iscell(Param) && ...
    length(Param) == size(bnet.G, 1) && ...
    all( ...
        arrayfun( @(i) ...
                isstruct(Param{i}) && ... % struct
                all(isfield(Param{i}, {'mu', 'b', 'sigma'})) && ... % required fields included
                isscalar(Param{i}.mu) && isreal(Param{i}.mu) && ... % valid mu
                isvector(Param{i}.b) && length(Param{i}.b) == length(parents(bnet, i)) && all(isreal(Param{i}.b)) && ... % valid b
                isscalar(Param{i}.sigma), ... % valid sigma
                1:length(Param)) ...
            );
        
assert( goodParam, ...
    'org:mensxmachina:pgm:GaussianBayesianNetwork:BadParam', ...
    ['PARAM must be a cell array of length equal to the number of ' ...
     'nodes in the network. Each cell must contain a struct with ' ...
     'fields ''mu'', ''b'' and ''sigma''. MU and SIGMA must be real ' ...
     'scalars. B must be a real vector of length equal to the number ' ...
     'of parents of the corresponding node.'] ...
    );

end

end


methods(Static)

% save / load

function obj = loadobj(s)

obj = org.mensxmachina.pgm.GaussianBayesianNetwork( ...
    s.G, s.NumLevels, s.Param, ...
    'VarNames', s.VarNames, ...
    'Labels', s.Labels, 'ClassNames', s.ClassNames, ...
    'Description', s.Description, 'UserData', s.UserData);

end

end

end