function bnet = dsl2pgm(nodes)
%DSL2PGM Convert DSL to PGM Bayesian network
%
%   BNET = DSL2PGM(NODES) converts Bayesian network NODES in the 
%   representation used in the Causal Explorer toolkit by the Discovery
%   Systems Laboratory (DSL) into the CategoricalBayesianNetwork object
%   BNET. The format of NODES is described in the Causal Explorer manual.
%   Causal Explorer can be downloaded from
%   http://www.dsl-lab.org/causal_explorer/index.html.
%
%   Example:
%
%       % read the Alarm network in the HUGIN file
%       alarm_nodes = hugin2dsl('alarm_h.net');
% 
%       % convert to PGM
%       alarm_bnet = org.mensxmachina.pgm.dsl2pgm(alarm_nodes);
%
%   See also: ORG.MENSXMACHINA.PGM.CATEGORICALBAYESIANNETWORK, 
%   ORG.MENSXMACHINA.PGM.BIFREAD, ORG.MENSXMACHINA.PGM.XDSLREAD

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% DSL network format:
% nodes{i}
%         .parents = Parents of the node i
%         .cpt     = Cumulative probability table
%         .name    = Name of the node

numNodes = numel(nodes);

% create graph and number of levels vector

row = [];
col = [];

numLevels = zeros(1, numNodes);

for j=1:numNodes % for each node
    
    row = [row nodes{j}.parents];
    col = [col j*ones(1, length(nodes{j}.parents))];
    
    numLevels(j) = size(nodes{j}.cpt, 1);
    
end

G = sparse(row, col, ones(size(row)), numNodes, numNodes);

Param  = cell(1, numNodes);

for j=1:numNodes
    
    if isempty(nodes{j}.parents)
        Param{j} = nodes{j}.cpt;
    else
        [parents_sorted parents_order] = sort(nodes{j}.parents);
        Param{j} = permute(nodes{j}.cpt, [ 1 parents_order+1 ]);
    end
    
    % fix probabilities so they sum to 1
    
    jParam_flat = Param{j}(:); % flatten CPD
    
    for k=numLevels(j):numLevels(j):length(jParam_flat)
        jParam_flat(k) = jParam_flat(k) - sum(jParam_flat((k-numLevels(j)+1):k)) + 1;
    end
    
    if ~isempty(nodes{j}.parents)
        Param{j} = reshape(jParam_flat, [numLevels(j) numLevels(parents_sorted)]);
    end
    
end

VarNames = cellfun(@(node) {node.name}, nodes);
Labels = arrayfun(@(j) arrayfun(@(k) sprintf('%s_%d', VarNames{j}, k), 1:numLevels(j), 'UniformOutput', false), 1:numNodes, 'UniformOutput', false);

% create the network object
bnet = org.mensxmachina.pgm.CategoricalBayesianNetwork(G, numLevels, Param, 'VarNames', VarNames, 'Labels', Labels, 'ClassNames', 'nominal');

end
