function tf = isdsep(G, c, b, a)
%ISDSEP Detect nodes d-separated from a given set of nodes by another
%   TF = ISDSEP(G, C, B, A) detects nodes of a set C of that are
%   d-separated from a set of nodes B given another set of nodes A in
%   graph G. G is an M-by-M sparse matrix that represents a directed graph.
%   Nonzero entries in matrix G indicate the presence of an edge. C is a
%   vector of positive integers. B and A are disjoint vectors of positive 
%   integers. TF is 1-by-M logical vector containing logical 1 (true) where
%   the nodes in C are d-separated from B given A and logical 0 (false) 
%   elsewhere.
%
%   See also ORG.MENSXMACHINA.PGM.ISDSEPOPT

%    References:
%       [1] Richard E. Neapolitan. Learning Bayesian Networks. Prentice
%       Hall, April 2003.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.graph.*;

assert(ndims(G) == 2 && size(G, 1) == size(G, 2) && issparse(G), ...
    'org:mensxmachina:pgm:isdsep:BadG', ...
    'G must be a M-by-M sparse matrix' ...
    );

assert((isvector(c) && all(org.mensxmachina.commons.ispositiveinteger(c))) || isempty(c), ...
    'org:mensxmachina:pgm:isdsep:BadC', ...
    'C must be a vector of positive integers or empty.' ...
    );

assert(isvector(b) && all(org.mensxmachina.commons.ispositiveinteger(b)), ...
    'org:mensxmachina:pgm:isdsep:BadB', ...
    'B must be a vector of positive integers.' ...
    );

assert((isvector(a) && all(org.mensxmachina.commons.ispositiveinteger(a))) || isempty(a), ...
    'org:mensxmachina:pgm:isdsep:BadA', ...
    'A must be a vector of positive integers or empty.' ...
    );

assert(isempty(intersect(b, a)), ...
    'org:mensxmachina:pgm:isdsep:BANotDisjoint', ...
    'B and A must be disjoint.' ...
    );

% get the set of all variables, V
v = 1:size(G, 1);

% create G', the graph with links in G going in both directions
G_prime = G + G';

% find parents in G ("inlist") and children in G ("outlist") of each node
parentsInG = arrayfun(@(j) parents(G, j), v, 'UniformOutput', false);
childrenInG = arrayfun(@(j) children(G, j), v, 'UniformOutput', false);

tf = org.mensxmachina.pgm.isdsepopt(G, G_prime, parentsInG, childrenInG, c, b, a);

end