function [skeleton p stat flag Stats] = mmpc(a, var, varargin)
%MMPC Learn parents and children using the MMPC algorithm
%   SKELETON = MMPC(A, VAR) learns the parents and children of the variable
%   of N-by-M dataset A specified by VAR by applying the non- symmetry
%   corrected MMPC algorithm to VAR and then to each of the learned parents
%   and children of VAR, performing hypothesis tests of conditional
%   independence at the (100*(ALPHA=0.05))% significance level. All the
%   learned edges are returned in the M-by-M sparse adjacency matrix 
%   SKELETON. Ones in SKELETON indicate the presence of an edge. A is a
%   dataset with categorical and / or numeric variables. VAR is a positive
%   integer or a variable name.
%
%   MMPC ignores rows with missing (undefined or NaN) values.
%
%   [SKELETON, P] = MMPC(A, VAR) also returns an M-by-M sparse matrix P 
%   containing upper bounds on the p-values of the complex hypotheses of 
%   absence of the edges in SKELETON. These bounds are the maximal p-values
%   among the tests of the simple hypotheses of conditional independence
%   between edge ends. P contains the maximal p-value of each edge and 0 
%   for each non-edge in SKELETON.
%
%   [SKELETON, P, STAT, FLAG] = MMPC(A) also returns a logical vector FLAG
%   of length M with which is true where the non- symmetry corrected MMPC
%   algorithm has been applied and false otherwise.
%
%   [SKELETON, P, STAT] = MMPC(A) also returns an M-by-M sparse matrix STAT
%   containing the statistics corresponding to the p-values in P.
%
%   [SKELETON, P, STAT, STATS] = MMPC(A, VAR) also returns a structure
%   STATS with the following fields:
%
%      'numTestsConsidered' An M-by-M sparse matrix containing the number
%                           of tests considered (whether they were 
%                           attempted or not) for each edge and zero for
%                           each non-edge in SKELETON.
%
%      'numTestsAttempted'  An M-by-M sparse matrix containing the number
%                           of tests attempted (whether they returned a 
%                           p-value or not) for each edge and zero for each
%                           non-edge in SKELETON.
%
%      'numTestsSucceeded'  An M-by-M sparse matrix containing the number
%                           of tests that successfully returned a p-value 
%                           for each edge and zero for each non-edge in
%                           SKELETON.
%
%   [...] = MMPC(A, VAR, 'Param1',VAL1,'Param2',VAL2,...) specifies
%   additional parameter name/value pairs chosen from the following:
%
%      'Alpha'              A value between 0 and 1 that specifies the
%                           significance level (100*ALPHA)% the hypothesis
%                           tests of conditional independence are performed
%                           at. Default is 0.05.
%
%      'Beta'               A value between 0 and 1 that specifies the
%                           threshold (100*(1 - BETA))% on the statistical
%                           power of the hypothesis tests of conditional
%                           independence. Tests with estimated power <
%                           (100*(1 - BETA)) are ignored. Default is 0.
%
%      'MaxK'               A non-negative integer that specifies the
%                           maximum cardinality of conditioning sets, or
%                           Inf. Tests with conditioning sets greater than
%                           MAXK are ignored. Default is 10.
%
%      'PowerFun'           A handle to a function that estimates the
%                           statistical power of a hypothesis test of
%                           conditional independence. Tests with estimated
%                           power < (100*(1 - BETA)) are ignored.
%                           Default is 
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWER.CHI2INDTESTHSPOPT
%                           when all variables are categorical,
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWER.DUMMYINDTESTPWROPT
%                           when all variables are numeric and 
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWER.INDTESTPWROPT otherwise.
%                           Type HELP ORG.MENSXMACHINA.PGM.INDTESTPOWER to
%                           see a list of available functions of this kind.
%                           Use 
%                           ORG.MENSXMACHINA.PGM.CUSTOM.TEMPLATE_INDTESTPOWER
%                           as a template for your own custom function.
%
%      'PowerMaxKFun'       A handle to a function that returns an upper
%                           bound on the maximum cardinality of a 
%                           conditioning set POWERFUN can be called with
%                           and return a power estimate >= 1 - BETA.
%                           POWERMAXKFUN is called before performing any
%                           hypothesis tests of conditional independence of
%                           two variables with subsets of the variables in
%                           TPC, in order to avoid subsequent calls of
%                           POWERFUN that are known to return a power
%                           estimate that is < 1 - BETA from this upper
%                           bound and above, thus improving performance.
%                           Default POWERMAXKFUN is
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.CHI2INDTESTHSPMAXKOPT
%                           when all variables are categorical, the
%                           dummy function 
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.DUMMYINDTESTPWRMAKOPT
%                           that returns the maximum possible cardinality 
%                           (i.e. it does not impose any restriction) when
%                           all variables are numeric and 
%                           ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK.INDTESTPWRMAXKOPT
%                           otherwise. Type
%                           HELP ORG.MENSXMACHINA.PGM.INDTESTPOWERMAXK to
%                           see a list of available functions of this kind.
%                           Use 
%                           ORG.MENSXMACHINA.PGM.CUSTOM.TEMPLATE_INDTESTPOWERMAXK
%                           as a template for your own custom function.
%
%      'PowerParam'         A structure of parameters passed to POWERFUN.
%                           Default is struct('Hps', 5) when all variables
%                           are categorical, an empty structure when all 
%                           variables are numeric and 
%                           struct('Chi2IndTestHspParam', struct('Hps', 5))
%                           otherwise.
%
%      'PFun'               A handle to a function that returns an estimate
%                           of the p-value and the statistic of a 
%                           hypothesis test of conditional independence.
%                           Default is 
%                           ORG.MENSXMACHINA.PGM.INDTESTP.CHI2TESTPOPT
%                           when all variables of A are categorical,
%                           ORG.MENSXMACHINA.PGM.INDTESTP.FISHERZINDTESTPOPT 
%                           when all variables are numeric and 
%                           ORG.MENSXMACHINA.PGM.INDTESTP.INDTESTPOPT
%                           otherwise. Type
%                           HELP ORG.MENSXMACHINA.PGM.INDTESTP to see a
%                           list of available functions of this kind. Use 
%                           ORG.MENSXMACHINA.PGM.CUSTOM.TEMPLATE_INDTESTP
%                           as a template for your own custom function.
%
%      'PParam'             A structure of parameters passed to PFUN or [].
%                           Default is struct('Stat', 2) when all variables
%                           are categorical, [] when all variables are
%                           numeric and 
%                           struct('Chi2IndTestHspParam', struct('Hps', 5))
%                           otherwise.
%
%      'TestParam'          A structure of parameters passed both to
%                           POWERFUN and PFUN, or []. Default is
%                           struct('Stat', 2) when all variables are
%                           categorical, [] when all variables are numeric
%                           and 
%                           struct('Chi2IndTestParam', struct('Stat', 2))
%                           otherwise.
%
%      'Verbose'            A logical scalar that specifies whether 
%                           messages are displayed during the execution of
%                           the function. Default is false.
%
%   Example: (categorical dataset)
%
%       % load ALARM network
%       load mxm_bnet_alarm_bnet; % loads 'bnet' CategoricalBayesianNetwork
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn parents and children set of variable #37
%       skeleton = org.mensxmachina.pgm.mmpc(a, 37);
%
%   See also ORG.MENSXMACHINA.PGM.MMPCNONSYM, 
%   ORG.MENSXMACHINA.PGM.MMPCSKELETON, ORG.MENSXMACHINA.PGM.MMPCOPT

%   References:
%       [1] Aliferis, C. F., Statnikov, A., Tsamardinos, I., Mani, S., and
%       Koutsoukos, X. D. 2010. Local Causal and Markov Blanket Induction 
%       for Causal Discovery and Feature Selection for Classification Part
%       I: Algorithms and Empirical Evaluation. J. Mach. Learn. Res. 11
%       (Mar. 2010), 171-234.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

persistent InstParam;

if isempty(InstParam)
    InstParam = mmpcgllinstparam();
end

% call GLLPC
[skeleton p stat flag Stats] = gllpc(a, var, InstParam, varargin{:});

end