function [skeleton p stat Stats] = mmpcskeleton(a, varargin)
%MMPCSKELETON Learn skeleton of a Bayesian Network using the MMPC algorithm
%   SKELETON = MMPCSKELETON(A) learns the skeleton of a Bayesian Network
%   from observations in N-by-M dataset A by applying the non- symmetry 
%   corrected MMPC algorithm to each of the variables and performing
%   hypothesis tests of conditional independence at the (100*(ALPHA=0.05))%
%   significance level. The learned skeleton is returned in the M-by-M
%   sparse adjacency matrix SKELETON. Ones in SKELETON indicate the
%   presence of an edge. A is a dataset with categorical and / or numeric
%   variables.
%
%   MMPCSKELETON ignores rows with missing (undefined or NaN) values.
%
%   [SKELETON, P] = MMPCSKELETON(A) also returns an M-by-M sparse matrix 
%   P containing upper bounds on the p-values of the complex hypotheses
%   of absence of the edges in SKELETON. These bounds are the maximal 
%   p-values among the tests of the simple hypotheses of conditional
%   independence between edge ends. P contains the maximal p-value of each
%   edge and 0 for each non-edge in SKELETON.
%
%   [SKELETON, P, STAT] = MMPCSKELETON(A) also returns an M-by-M
%   sparse matrix STAT containing the statistics corresponding to the
%   p-values in P.
%
%   [SKELETON, P, STAT, STATS] = MMPCSKELETON(A) also returns a structure
%   STATS with the following fields:
%
%      'numTestsConsidered' An M-by-M sparse matrix containing the number
%                           of tests considered (whether they were 
%                           attempted or not) for each edge and zero for
%                           each non-edge in SKELETON.
%
%      'numTestsAttempted'  An M-by-M sparse matrix containing the number
%                           of tests attempted (whether they returned a 
%                           p-value or not) for each edge and zero for each
%                           non-edge in SKELETON.
%
%      'numTestsSucceeded'  An M-by-M sparse matrix containing the number
%                           of tests that successfully returned a p-value 
%                           for each edge and zero for each non-edge in
%                           SKELETON.
%
%   [...] = MMPCSKELETON(A, 'Param1', VAL1, 'Param2', VAL2,...) specifies
%   the additional parameter name/value pairs supported by
%   ORG.MENSXMACHINA.PGM.MMPC.
%
%   Example 1: (categorical dataset)
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn ALARM's skeleton
%       alarm_skeleton = org.mensxmachina.pgm.mmpcskeleton(a);
%
%   Example 2: (continuous dataset)
%
%       % load Smoking and Cancer dataset by J.F. Fraumeni downloaded from
%       % http://www.phil.cmu.edu/projects/tetrad_examples/ (Example 7)
%       a = dataset( ...
%            'File', 'smokingandcancer.txt',
%            'ReadObsNames', false);
%
%       % learn skeleton
%       smokingandcancer_skeleton = org.mensxmachina.pgm.mmpcskeleton(a);
%
%   See also
%       ORG.MENSXMACHINA.PGM.MMPCNONSYM, ORG.MENSXMACHINA.PGM.MMPC

%   References:
%       [1] Aliferis, C. F., Statnikov, A., Tsamardinos, I., Mani, S., and
%       Koutsoukos, X. D. 2010. Local Causal and Markov Blanket Induction 
%       for Causal Discovery and Feature Selection for Classification Part
%       I: Algorithms and Empirical Evaluation. J. Mach. Learn. Res. 11
%       (Mar. 2010), 171-234.

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

persistent InstParam;

if isempty(InstParam)
    InstParam = mmpcgllinstparam();
end

[skeleton p stat Stats] = lglskeleton(a, InstParam, varargin{:});

end