function [tpr fpr threshold] = pcroc(targetskeleton, p, t)
%PCROC Parents and children learning receiver operating characteristic
%
%   PCROC is a version of the ORG.MENSXMACHINA.COMMONS.STATS.ROC
%   function specialized for parents and children learning. For a given
%   node, the task of learning its parents and children can be viewed as a
%   binary classification task, where each other node is classified as
%   either parent or child, or not.
%
%   [TPR,FPR,THRESHOLD] = PCROC(TARGETSKELETON, P, T), where P is the
%   maximum p-values vector returned by either ORG.MENSXMACHINA.PGM.MMPC
%   (with 'includeNpc' parameter set to true) or
%   ORG.MENSXMACHINA.PGM.MMPCBAR (with 'includeNpc' parameter set to
%   true) targeting the variable in adjacency matrix TARGETSKELETON 
%   specified by the positive integer T, calculates for each threshold
%   (here the threshold are the negative maximum p-values) the true 
%   positive and false positive rate of the parents and children learner 
%   and returns them in row vectors TPR and FPR, while the corresponding
%   threshold are returned in the THRESHOLD row vector. Each pair of 
%   values (TPR(k), FPR(k)) defines a point in the Receiver operating
%   characteristic (ROC) curve.
%
%   Example:
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' BayesianNetwork
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn parents and children set of variable #37
%       [pc p stat] = ...
%           org.mensxmachina.pgm.mmpc(a, 37, 'includeNpc', true);
%
%       % get true skeleton
%       skeleton_true = net.skeleton;
%
%       % calculate ROC for variable #37
%       [tpr fpr threshold] = ..
%           org.mensxmachina.pgm.pcroc(skeleton_true, p, 37);
%
%   See also: 
%       ORG.MENSXMACHINA.COMMONS.STATS.ROC,
%       ORG.MENSXMACHINA.PGM.PLOTPCROC,
%       ORG.MENSXMACHINA.PGM.PCAUC 

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

% get roc inputs
[targets outputs] = rocinputs(targetskeleton, p, t);

% calculate ROC
[tpr fpr threshold] = ...
    org.mensxmachina.commons.stats.roc(targets, outputs);

end

function [targets outputs] = rocinputs(targetskeleton, p, t)

targets = targetskeleton(t, :);
targets(t) = []; % remove target variable

outputs = -p;
outputs(t) = []; % remove target variable

end