function r = skeletonnp(outputSkeleton, p, varargin)
%SKELETONNP Number of discovered undirected edges (positives)
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   R = SKELETONNP(OUTPUTSKELETON, P) calculates the number R of discovered
%   undirected edges (positives) corresponding to undirected edges in 
%   output skeleton OUTPUTSKELETON with p-values P. The number of 
%   discovered undirected edges corresponding to an undirected edge is the
%   one obtained when keeping only the undirected edges with  p-values less
%   than or equal to the p-value of that undirected edge and throwing the
%   rest away. OUTPUTSKELETON is a M-by-M sparse adjacency matrix that
%   represents the skeleton, i.e. the graph with undirected edges in place
%   of directed edges, of a PGM's graph. P is an M-by-M sparse matrix 
%   containing the p-values of the null hypotheses of absence of the edges
%   in OUTPUTSKELETON. P contains the p-value of each edge and 0 for each
%   non-edge in OUTPUTSKELETON. V is an M-by-M sparse matrix that contains
%   the number of falsely discovered undirected edges of each edge and 0
%   for each non-edge in OUTPUTSKELETON.
%
%   R = SKELETONNP(OUTPUTSKELETON, P, T) calculates the number R of 
%   positives corresponding to each p-value threshold in T. T is a numeric
%   real vector. R is a vector of the same size as T.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.NP,
%   ORG.MENSXMACHINA.PGM.SKELETONNFP, ORG.MENSXMACHINA.PGM.SKELETONRLZDFPR, 
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDFDR, 
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDPWR  

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(outputSkeleton) && issquarematrix(outputSkeleton), ...
        'org:menxmachina:pgm:skeletonnp:BadOutputSkeleton', ...
        'OUTPUTSKELETON must an M-by-M sparse matrix.');

assert( issparse(p) && isnumeric(p), ...
        'org:menxmachina:pgm:skeletonnp:NonNumericP', ...
        'P must be numeric.');

assert( isequal(size(p), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonnp:PSizeNotOutputSkeletonSize', ...
        'P must a matrix of the same size as OUTPUTSKELETON.');

% convert output skeleton to vector
outputSkeleton_vector = skeleton2vector(outputSkeleton);

% convert maximum p-value matrix to vector
p_vector = skeleton2vector(p);

% get edge linear indices in output skeleton vector
outputSkeleton_vector_ind = find(outputSkeleton_vector);

% keep only maximum p-values corresponding to edges and convert to full
p_outputSkeleton_vector = full(p_vector(outputSkeleton_vector_ind));

% compute realized FDR corresponding to these edges
r_outputSkeleton_vector = np(p_outputSkeleton_vector, varargin{:});

if nargin == 2

    % create output skeleton realized FDR vector
    r_vector = sparse(outputSkeleton_vector_ind, 1, r_outputSkeleton_vector, length(outputSkeleton_vector), 1);

    % convert realized FDR vector to matrix
    r = vector2skeleton(r_vector);

else % thresholds given
    
    r = r_outputSkeleton_vector;
    
end

end