function cp = skeletonperf(targetskeleton, outputskeleton)
%SKELETONPERF Evaluate performance of skeleton learners
%   SKELETONPERF is a version of the CLASSPERF function in the
%   Bioinformatics Toolbox (TM), specialized for skeleton identification.
%   A skeleton identification task can be viewed as a binary classification
%   task, where the possible edges are classified as edges (positives) or
%   non-edges (negatives).
%   
%   CP = SKELETONPERF(TARGETSKELETON, OUTPUTSKELETON) creates a
%   classifier performance object CP with the performance of a skeleton 
%   learner with output OUTPUTSKELETON targeting the true skeleton 
%   TARGETSKELETON.
%
%   Example:
%
%       % load ALARM network samples
%       load mxm_bnet_alarm_sample; % loads 'a' dataset
%
%       % learn skeleton
%       skeleton = org.mensxmachina.pgm.mmpcskeleton(a);
%
%       % load ALARM network
%       load mxm_bnet_alarm_net; % loads 'net' network
%
%       % get true skeleton
%       skeleton_true = net.skeleton;
%
%       % get classifier performance object
%       cp = org.mensxmachina.pgm.skeletonperf(skeleton_true, skeleton);
%
%   See also: CLASSPERF

%   Copyright 2010-2011 Mens X Machina
%
%   This file is part of Mens X Machina Probabilistic Graphical Model
%   Toolbox.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is free software:
%   you can redistribute it and/or modify it under the terms of the GNU 
%   General Public License as published by the Free Software Foundation,
%   either version 3 of the License, or (at your option) any later version.
%
%   Mens X Machina Probabilistic Graphical Model Toolbox is distributed in
%   the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
%   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
%   PURPOSE. See the GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with Mens X Machina Probabilistic Graphical Model Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

skeletonvector_true = double(org.mensxmachina.commons.graph.skeleton2vector(targetskeleton));
skeletonvector = double(org.mensxmachina.commons.graph.skeleton2vector(outputskeleton));

% get skeleton learning performance
cp = classperf(skeletonvector_true, skeletonvector, 'Positive', 1, 'Negative', 0); 

end