function y = skeletonrlzdfdr(outputSkeleton, p, targetSkeleton, varargin)
%SKELETONRLZDFDR Realized False Discovery Rate (FDR) of skeleton identification
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   Y = SKELETONRLZDFDR(OUTPUTSKELETON, P, TARGETSKELETON) calculates the
%   realized False Discovery Rate (FDR) Y corresponding to undirected edges
%   in OUTPUTSKELETON with p-values P, given the target skeleton
%   TARGETSKELETON. The realized FDR of an undirected edge is the ratio V/R
%   of the number V of falsely discovered undirected edges to the number R
%   of discovered undirected edges when keeping only the undirected edges
%   with p-values less than or equal to the p-value of that undirected edge
%   and throwing the rest away. OUTPUTSKELETON and TARGETSKELETON are 
%   M-by-M sparse adjacency matrices that represent the skeleton, i.e. the
%   graph with undirected edges in place of directed edges, of a PGM's
%   graph. P is an M-by-M sparse matrix containing the p-values of the null
%   hypotheses of absence of the edges in OUTPUTSKELETON. P contains the
%   p-value of each edge and 0 for each non-edge in OUTPUTSKELETON. Y is an
%   M-by-M sparse matrix that contains the realized FDR of each edge and 0
%   for each non-edge in OUTPUTSKELETON.
%
%   Y = SKELETONRLZDFDR(OUTPUTSKELETON, P, TARGETSKELETON, T) calculates
%   the realized FDR Y corresponding to each p-value threshold in T. T is a
%   numeric real vector. Y is a vector of the same size as T.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.RLZDFDR, 
%   ORG.MENSXMACHINA.PGM.SKELETONNFP, ORG.MENSXMACHINA.PGM.SKELETONNP, 
%   ORG.MENSXMACHINA.PGM.SKELETONRLZDFPR

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(outputSkeleton) && issquarematrix(outputSkeleton), ...
        'org:menxmachina:pgm:skeletonrlzdfdr:BadOutputSkeleton', ...
        'SKELETON must an M-by-M sparse matrix.');

assert( issparse(p) && isnumeric(p), ...
        'org:menxmachina:pgm:skeletonrlzdfdr:NonNumericP', ...
        'P must be numeric.');

assert( isequal(size(p), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonrlzdfdr:PSizeNotOutputSkeletonSize', ...
        'P must a matrix of the same size as SKELETON.');

assert( isequal(size(targetSkeleton), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonrlzdfdr:TrueSkeletonSizeNotOutputSkeletonSize', ...
        'TRUESKELETON must be of the same size as SKELETON.');

% convert output skeleton to vector
outputSkeleton_vector = skeleton2vector(outputSkeleton);

% convert maximum p-value matrix to vector
p_vector = skeleton2vector(p);

% convert true output skeleton to vector
targetSkeleton_vector = skeleton2vector(targetSkeleton);

% get edge linear indices in output skeleton vector
outputSkeleton_vector_ind = find(outputSkeleton_vector);

% keep only maximum p-values corresponding to edges and convert to full
p_outputSkeleton_vector = full(p_vector(outputSkeleton_vector_ind));

% keep only possible edges that are edges in output skeleton and convert to full
targetSkeleton_outputSkeleton_vector = full(targetSkeleton_vector(outputSkeleton_vector_ind));

% compute realized FDR corresponding to these edges
y_skeleton_vector = rlzdfdr(p_outputSkeleton_vector, targetSkeleton_outputSkeleton_vector, varargin{:});

if nargin == 3

    % create output skeleton realized FDR vector
    y_vector = sparse(outputSkeleton_vector_ind, 1, y_skeleton_vector, length(outputSkeleton_vector), 1);

    % convert realized FDR vector to matrix
    y = vector2skeleton(y_vector);

else % thresholds given
    
    y = y_skeleton_vector;
    
end

end