function y = skeletonrlzdpwr(skeleton, p, trueSkeleton, varargin)
%SKELETONRLZDPWR Skeleton identification realized power
%   SKELETONSTEPUP is a wrapper for ORG.MENSXMACHINA.COMMONS.STATS.STEPUP
%   specialized for Probabilistic Graphical Model (PGM) output skeleton
%   identification. Skeleton identification is viewed as a multiple testing
%   procedure, the null hypotheses being the absence of undirected edges.
%
%   Y = SKELETONRLZDPWR(OUTPUTSKELETON, P, TARGETSKELETON) calculates the
%   realized power Y corresponding to undirected edges in OUTPUTSKELETON 
%   with p-values P, given the target skeleton TARGETSKELETON. The realized
%   power of an undirected edge is the ratio S/M1 of the number S of
%   discovered true undirected edges to the number M1 of true undirected
%   edges when keeping only the undirected edges with p-values less than or
%   equal to the p-value of that undirected edge and throwing the rest
%   away. OUTPUTSKELETON and TARGETSKELETON are M-by-M sparse adjacency
%   matrices that represent the skeleton, i.e. the graph with undirected 
%   edges in place of directed edges, of a PGM's graph. P is an M-by-M 
%   sparse matrix containing the p-values of the null hypotheses of absence
%   of the edges in OUTPUTSKELETON. P contains the p-value of each edge and
%   0 for each non-edge in OUTPUTSKELETON. Y is an M-by-M sparse matrix
%   that contains the realized power of each edge and 0 for each non-edge
%   in OUTPUTSKELETON.
%
%   Y = SKELETONRLZDPWR(OUTPUTSKELETON, P, TARGETSKELETON, T) calculates
%   the realized power Y corresponding to each p-value threshold in T. T is
%   a numeric real vector. Y is a vector of the same size as T.
%
%   See also ORG.MENSXMACHINA.COMMONS.STATS.RLZDPWR, 
%   ORG.MENSXMACHINA.PGM.SKELETONNFP, ORG.MENSXMACHINA.PGM.SKELETONNP

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(skeleton) && issquarematrix(skeleton), ...
        'org:menxmachina:pgm:skeletonfdrcontrol:BadSkeleton', ...
        'SKELETON must an M-by-M sparse matrix.');

assert( issparse(p) && isequal(size(p), size(skeleton)), ...
        'org:menxmachina:pgm:skeletonfdrcontrol:BadP', ...
        'P must a matrix of the same size as SKELETON.');

assert( isequal(size(trueSkeleton), size(skeleton)), ...
        'org:menxmachina:pgm:skeletonfdr:TrueSkeletonSizeNotSkeletonSize', ...
        'TRUESKELETON must be of the same size as SKELETON.');

% convert skeleton to vector
skeleton_vector = skeleton2vector(skeleton);

% convert maximum p-value matrix to vector
p_vector = skeleton2vector(p);

% convert true skeleton to vector
trueSkeleton_vector = skeleton2vector(trueSkeleton);

% get edge linear indices in skeleton vector
skeleton_vector_ind = find(skeleton_vector);

% keep only maximum p-values corresponding to edges and convert to full
p_skeleton_vector = full(p_vector(skeleton_vector_ind));

% keep only possible edges that are edges in skeleton and convert to full
trueSkeleton_skeleton_vector = full(trueSkeleton_vector(skeleton_vector_ind));

% number of alternative hypotheses = number of true edges
m1 = nnz(trueSkeleton_vector);

% compute realized power corresponding to these edges
y_skeleton_vector = rlzdpwr(p_skeleton_vector, trueSkeleton_skeleton_vector, varargin{:}, 'M1', m1);

if nargin == 3

    % create a vector of the same size as the original skeleton vector and
    % fill the elements corresponding to edges with the corresponding
    % elements in the power vector
    y_vector = sparse(skeleton_vector_ind, 1, y_skeleton_vector, length(skeleton_vector), 1);

    % convert realized power vector to matrix
    y = vector2skeleton(y_vector);
    
else % thresholds given
    
    y = y_skeleton_vector;
    
end

end