function y = skeletonrlzdtsr(outputSkeleton, numTestsSucceeded, numTestsConsidered)
%SKELETONRLZDTSR Realized Test Success Rate (TSR) of constraint-based skeleton identification
%   Y = SKELETONRLZDTSR(SKELETON, NTS, NTC) calculates the realized Test
%   Success Rate (TSR) Y corresponding to undirected edges in
%   OUTPUTSKELETON with p-values P, number of hypothesis tests of 
%   conditional independence succeeded NTS and number of tests considered 
%   NTS. The realized TSR of an undirected edge is the ratio of the number
%   of tests succeeded to the number of tests considered by the 
%   constraint-based skeleton identification function that generated 
%   OUTPUTSKELETON. OUTPUTSKELETON is a M-by-M sparse adjacency matrix that
%   represent the skeleton, i.e. the graph with undirected edges in place
%   of directed edges, of a PGM's graph. NTS and NTC are M-by-M sparse
%   matrices containing the number of tests succeeded and the number of
%   tests considered for each edge and 0 for each non-edge in 
%   OUTPUTSKELETON. Y is an M-by-M sparse matrix that contains the realized
%   TSR of each edge and 0 for each non-edge in OUTPUTSKELETON.

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.graph.*;
import org.mensxmachina.commons.stats.*;

assert( issparse(outputSkeleton) && issquarematrix(outputSkeleton), ...
        'org:menxmachina:pgm:skeletonrlzdtsr:BadSkeleton', ...
        'SKELETON must an M-by-M sparse matrix.');

assert( issparse(numTestsSucceeded) && isnumeric(numTestsSucceeded), ...
        'org:menxmachina:pgm:skeletonrlzdtsr:NonNumericNumTestsSucceeded', ...
        'NUMTESTSSUCCEEDED must be sparse and numeric.');

assert( issparse(numTestsConsidered) && isnumeric(numTestsConsidered), ...
        'org:menxmachina:pgm:skeletonrlzdtsr:NonNumericNumTestsConsidered', ...
        'NUMTESTSCONSIDERED must be sparse and numeric.');

assert( isequal(size(numTestsSucceeded), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonrlzdtsr:NumTestsSucceededSizeNotSkeletonSize', ...
        'NUMTESTSSUCCEEDED must a matrix of the same size as SKELETON.');

assert( isequal(size(numTestsConsidered), size(outputSkeleton)), ...
        'org:menxmachina:pgm:skeletonrlzdtsr:NumTestsConsideredSizeNotSkeletonSize', ...
        'NUMTESTSCONSIDERED must a matrix of the same size as SKELETON.');

% convert skeleton to vector
skeleton_vector = skeleton2vector(outputSkeleton);

% convert num tests succeeded matrix to vector
numTestsSucceeded_vector = skeleton2vector(numTestsSucceeded);

% convert num tests considered matrix to vector
numTestsConsidered_vector = skeleton2vector(numTestsConsidered);

% get edge linear indices in skeleton vector
skeleton_vector_ind = find(skeleton_vector);

% keep only num tests succeeded corresponding to edges and convert to full
numTestsSucceeded_skeleton_vector = full(numTestsSucceeded_vector(skeleton_vector_ind));

% keep only num tests considered corresponding to edges and convert to full
numTestsConsidered_skeleton_vector = full(numTestsConsidered_vector(skeleton_vector_ind));

% compute realized TSR corresponding to these edges
y_skeleton_vector = numTestsSucceeded_skeleton_vector ./ numTestsConsidered_skeleton_vector;

% create skeleton realized TSR vector
y_vector = sparse(skeleton_vector_ind, 1, y_skeleton_vector, length(skeleton_vector), 1);

% convert realized TSR vector to matrix
y = vector2skeleton(y_vector);

end